{* Matrix Made Easy                                             *}
{* Author: Andrew N. Tuseyev                                    *}
{* Version 0.2b                                                 *}
{* Description: Complex matrix class and function of            *}
{*              complex variable                                *}
{* If you have some idea contact with me                        *}
{* E-Mail: brain@uti.ru                                         *}
{* All file provided "as is" without  warranties of stable work *}
{*--------------------------------------------------------------*}
{* If this library was useful to you send me some something by  *}
{* mail (like postcard, postmark, map of your city and etc)     *}
{* Thanx a lot.                                                 *}

unit MMatrix;

interface

uses Classes,Math,SysUtils,Grids,mcomplex;

type PMatrixItem = ^Tcomplex;

type
  TMatrix=class(TObject)
  private

   Acol:integer;
   Arow:integer;
   MatrixItem:PMatrixItem;
   AItems:Tlist;
   ErrorLog:integer;
   ReverseCount:integer;
   ReverseBit:boolean;
   FCheckPrecision:Boolean;

   function  getvalues(x,y:integer):Tcomplex;
   function  GetDeterminant:Tcomplex;
   procedure setvalues(x,y:integer;value:Tcomplex);
   procedure AddZeroItem;
   function  GetCheckPrecision:Boolean;
   procedure SetCheckPrecision(value:Boolean);
   function  GetFootStep:TComplex;

  public

   constructor create(x,y:integer);
   destructor Destroy;override;


   property Items[x,y:integer]:Tcomplex read getvalues write setvalues; default;
   property ColCount:integer read acol;//getcolcount;
   property RowCount:integer read arow;//getrowcount;
   property Determinant:Tcomplex read GetDeterminant; //   ( ,  )
   property CheckPrecision:Boolean read GetCheckPrecision write SetCheckPrecision;     // 
   Property FootStep:Tcomplex read GetFootStep; //  

   {Procedures and Finction}
   procedure AddRow;                                    //  
   procedure InsertRow(x:integer);                      //   ...
   Procedure DeleteRow(x:integer);                      //  
   procedure AddCOl;                                    //   
   procedure InsertCol(x:integer);                      // ..
   procedure DeleteCol(x:integer);                      // ..

   procedure Exchange(x,y,x2,y2:integer);               //     
   procedure Transposition;                             //  (  )
   procedure LowTriangulation;                          //     ( )
   procedure HighTriangulation;                         // ........  .....
   procedure InvertMatrix;                              //  
   procedure RecreateMatrix(x,y:integer);               //      (, ,  )
   {Math Crazy}
   procedure AddMatrix(m:Tmatrix);                      //    
   procedure SubMatrix(m:Tmatrix);                      //   
   procedure MulMatrixPerComplex(z:Tcomplex);           //     
   procedure DivMatrixPerComplex(z:Tcomplex);           //  .....
   {crazy Process}
   procedure FillMatrix(mode:integer);                  //     . 
   procedure FillMatrixWithValue(z:tcomplex);
   procedure CopyFromMatrix(z:Tmatrix);                 //   (  )    Z
   procedure CopyRealFromMatrix(z:Tmatrix);
   procedure CopyImagineFromMatrix(z:Tmatrix);
   procedure FlipVertical;                              //  
   Procedure FlipHorizontal;                            // .... 
   Procedure FillMatrixFromArray(const A:array of Tcomplex); //       
   Procedure FillMatrixFromArrayE(const A:array of double);
   Procedure FillGridWithMatrix(Grid:TStringGrid;x,y:integer); //    StringGrid'
   Function MaxModuleNonDiagonalItem(var x,y:integer):TComplex; //        
   Procedure RotateMatrix(Angle:double;mx,my:integer);
   procedure InvertSign;
   procedure TakeMatrixPart(x,y,x1,y1:integer;var A:TMatrix);
   procedure PlaceMatrixPart(A:TMatrix;x,y:integer);
   procedure ExchangeCols(x1,x2:integer);
   procedure ExchangeRows(y1,y2:integer);

   function Norma:Double;
   procedure SaveToStream(AStream:TStream);
   procedure SaveToFile(Name:string);
   procedure LoadFromStream(AStream:TStream);
   procedure LoadFromFile(Name:string);
end;

// Matrix Operation

function EqualSizeMatrix(m1,m2:TMatrix):boolean;

function AddMatrix(m1,m2:TMatrix):Tmatrix;
function SubMatrix(m1,m2:TMatrix):TMatrix;
function MulMatrix(m1,m2:TMatrix):TMatrix;
function MulMatrixPerComplex(m1:Tmatrix;z:tcomplex):TMatrix;
function DivMatrixPerComplex(m1:Tmatrix;z:tcomplex):TMatrix;

procedure AddMatrixEx(m1,m2:Tmatrix; var r:Tmatrix);
procedure SubMatrixEx(m1,m2:Tmatrix; var r:Tmatrix);
procedure MulMatrixEx(m1,m2:Tmatrix; var r:Tmatrix);

procedure DeterminateLW(X:Tmatrix; var L,W:TMatrix);
procedure SolveMatrixL(L,B:Tmatrix;var Z:Tmatrix);
procedure SolveMatrixW(W,B:Tmatrix;var Z:Tmatrix); //      
procedure GaussInvert(A:TMatrix;var B:TMatrix);


implementation

const HeaderCode=$2552;
      VersionCode=$0101;
      LabelCode='MME';

type TFileHeader=packed record
     Lbl:string[3];
     Header:Word;
     Version:Word;
     FCol,FRow:LongInt;
end;

// System matrix operation


function EqualSizeMatrix(m1,m2:TMatrix):boolean;
begin
 result := (m1.colcount=m2.colcount) and (m1.rowcount=m2.rowcount);
end;

//Matrix Operation

procedure DeterminateLW(X:Tmatrix; var L,W:TMatrix);
var k,j,i,x1:integer;
//    x:tcomplex;
begin
if not EqualSizeMatrix(X,L) then  L.RecreateMatrix(X.colcount,X.rowcount);
if not EqualSizeMatrix(X,W) then  W.RecreateMatrix(X.colcount,X.rowcount);

W.CopyFromMatrix(X);
{f  acol<arow then
  begin
  errorlog:=12; //    -    (    )
  exit;
  end;}
for k:=1 to X.RowCount do
 begin
{  if ComplexEqualZero(Items[k,k]) then //    =0  
    begin
    x1:=k+1;
    while (x1<Arow) and (ComplexEqualZero(Items[k,x1])) do inc(x1);     //    
    if (x1=Arow) and (ComplexEqualZero(Items[k,x1])) then               //     
      for i:=1 to ARow do exchange(k,i,k+1,i)
    else
      for i:=1 to ACol do exchange(i,k,i,x1);
    end;}

  L[k,k]:=MakeComplex(1,0);
  for j:=k+1 to X.RowCount do
   begin
   L[k,j]:=DivComplex(W[k,j],W[k,k]);
//   x:=DivComplex(NegativeCOmplex(Items[k,j]),Items[k,k]);
//   W[i,j]-L[k,j]*W[i,j-1]
   for i:=k to X.ColCount do
      W[i,j]:=SubComplex( W[i,j] , MulComplex(L[k,j],W[i,k]) );
//   addcomplex(mulcomplex(Items[i,k],x),Items[i,j]);
   end;

 end;
end;

procedure AddMatrixEx(m1,m2:Tmatrix; var r:Tmatrix);
var i,j:integer;
begin
if EqualSizeMatrix(m1,m2) then
  begin
   if not EqualSizeMatrix(r,m1) then  r.RecreateMatrix(m1.colcount,m2.rowcount);
   for j:=1 to m1.Rowcount do for i:=1 to m1.ColCount do r[i,j]:=AddComplex(m1[i,j],m2[i,j]);
  end
else  Raise EMathError.Create('Operation can''t complete because first matrix column count not equal second matrix row count.');
end;

procedure SubMatrixEx(m1,m2:Tmatrix; var r:Tmatrix);
var i,j:integer;
begin
if EqualSizeMatrix(m1,m2) then
  begin
   if not EqualSizeMatrix(r,m1) then  r.RecreateMatrix(m1.colcount,m2.rowcount);
   for j:=1 to m1.Rowcount do for i:=1 to m1.ColCount do r[i,j]:=SubComplex(m1[i,j],m2[i,j]);
  end
else Raise EMathError.Create('Operation can''t complete because first matrix column count not equal second matrix row count.');
end;

procedure MulMatrixEx(m1,m2:Tmatrix; var r:Tmatrix);
var i,j,k:integer;
begin
if m1.colcount=m2.rowcount then
  begin
   if not EqualSizeMatrix(r,m2) then r.RecreateMatrix(m2.ColCount,m1.RowCount);
   for j:=1 to R.RowCount do for i:=1 to R.Colcount do for k:=1 to m1.colcount
    do
     r[i,j]:=AddComplex( r[i,j] , MulComplex( m1[k,j],m2[i,k] ) );

  end
else Raise EMathError.Create('Operation can''t complete because first matrix column count not equal second matrix row count.');
end;

function AddMatrix(m1,m2:TMatrix):TMatrix;
var i,j:integer;
begin
Result:=TMatrix.Create(m1.ColCount,m1.rowCount);
for j:=1 to m1.Rowcount do
 for i:=1 to m1.ColCount do
  result[i,j]:=AddComplex(m1[i,j],m2[i,j]);
end;

function SubMatrix(m1,m2:TMatrix):TMatrix;
var i,j:integer;
begin
Result:=TMatrix.Create(m1.ColCount,m1.rowCount);
for j:=1 to m1.Rowcount do
 for i:=1 to m1.ColCount do
  result[i,j]:=SubComplex(m1[i,j],m2[i,j]);
end;

function MulMatrix(m1,m2:TMatrix):TMatrix;
var i,j,k:integer;
begin
if m1.colcount=m2.rowcount then
 begin
 Result:=TMatrix.Create(m2.ColCount,m1.rowCount);
 for j:=1 to Result.RowCount do for i:=1 to Result.Colcount do for k:=1 to m1.colcount
  do result[i,j]:=AddComplex(result[i,j],MulComplex(m1[k,j],m2[i,k]))
 end
else
 Raise EMathError.Create('Operation can''t complete because first matrix column count not equal second matrix row count.');
end;

function MulMatrixPerComplex(m1:Tmatrix;z:tcomplex):TMatrix;
var i,j:integer;
begin
Result:=TMatrix.Create(m1.ColCount,m1.rowCount);
for j:=1 to m1.Rowcount do
 for i:=1 to m1.ColCount do
  result[i,j]:=MulComplex(m1[i,j],z);
end;

function DivMatrixPerComplex(m1:Tmatrix;z:tcomplex):TMatrix;
var i,j:integer;
begin
if not ComplexEqualZero(z) then
 begin
 Result:=TMatrix.Create(m1.ColCount,m1.rowCount);
 for j:=1 to m1.Rowcount do for i:=1 to m1.ColCount do result[i,j]:=DivComplex(m1[i,j],z);
 end
else raise EDivByZero.Create('Dividyig By Zero.');
end;

// Inside Matrix Class

procedure TMatrix.AddZeroItem;
begin
 new(MatrixItem);
 MatrixItem:=Nil;
 AItems.add(MatrixItem);
end;

constructor TMatrix.create(x,y:integer);
var i:integer;
begin
Acol:=x;
ARow:=y;
AItems:=Tlist.create;
AItems.clear;
AItems.capacity:=Acol*Arow;
for i:=0 to Acol*Arow-1 do AddZeroItem;
inherited Create;
end;

destructor TMatrix.Destroy;
var i:integer;
begin
for i:=0 to AItems.count-1 do
 begin
 MatrixItem:=AItems.items[i];
 dispose(MatrixItem);
 end;
AItems.free;
inherited destroy;
end;

function TMatrix.getvalues(x,y:integer):Tcomplex;
begin
if ((x>0) and (x<=colcount)) and ((y>0) and (y<=rowcount)) then
 begin
 MatrixItem:=AItems.items[(y-1)*Acol+x-1];
 if MatrixItem=Nil then Result:=NullComplex else result:=Matrixitem^;
 end;
end;

procedure TMatrix.setvalues(x,y:integer;value:Tcomplex);
begin
if ((x>0) and (x<=colcount)) and ((y>0) and (y<=rowcount)) then
 begin
 MatrixItem:=AItems.items[(y-1)*Acol+x-1];
 if ComplexEqualZero(Value) then
   begin
   dispose(MatrixItem);
   AItems.items[(y-1)*Acol+x-1]:=nil;
   end
 else
  begin
 if MatrixItem=nil then new(MatrixItem);
  if FCheckPrecision=true then if Value.r<0.00000000000000000001 then Value.r:=0;
  Matrixitem^:=value;
  AItems.items[(y-1)*Acol+x-1]:=MatrixItem;
  end;
 end;
end;

procedure TMatrix.AddRow;
var i:integer;
begin
inc(Arow);
AItems.capacity:=Acol*Arow;
for i:=1 to Acol do AddZeroItem;
end;

procedure TMatrix.InsertRow(x:integer);
var i:integer;
begin
if x<=ColCount then
 begin
 inc(Arow);
 AItems.capacity:=Acol*Arow;
 for i:=1 to Acol do
  begin
  new(MatrixItem);
  MatrixItem:=nil;
  AItems.Insert(Acol*x,MatrixItem);
  end;
 end;
end;

Procedure TMAtrix.DeleteRow(x:integer);
var i:integer;
begin
if (x<=RowCount) and (x>0) then
 begin
 dec(Arow);
 for i:=1 to Acol do AItems.delete(Acol*(x-1));
 AItems.capacity:=Acol*Arow;
 end;
end;

procedure TMatrix.AddCol;
var i:integer;
begin
inc(Acol);
AItems.capacity:=Acol*Arow;
for i:=1 to Arow-1 do
 begin
 new(MatrixItem);
 MatrixItem:=nil;
 AItems.Insert(Acol*i-1,MatrixItem);
 end;
AddZeroItem;
end;

procedure TMatrix.InsertCol(x:integer);
Var I:integer;
begin
inc(Acol);
AItems.capacity:=Acol*Arow;
for i:=1 to Arow do
 begin
 new(MatrixItem);
 MatrixItem:=nil;
 AItems.Insert(Acol*i-(acol-x),MatrixItem);
 end;
end;

procedure TMatrix.DeleteCol(x:integer);
Var I:integer;
begin
dec(x);
for i:=Arow downto 1 do  AItems.Delete(Acol*i-(acol-x));
dec(Acol);
AItems.capacity:=Acol*Arow;
end;

procedure TMatrix.Exchange(x,y,x2,y2:integer);
begin
AItems.Exchange((Acol*(y-1)+x)-1,(Acol*(y2-1)+x2)-1);
end;

procedure TMatrix.Transposition;
var i,j,x,y,z:integer;

procedure SwapInteger(var x, y: Integer); assembler;
asm
  mov  ecx, [EAX];
  xchg ecx, [EDX]
  mov  [EAX], ecx
end;

begin
x:=acol;
y:=arow;
if (x=1) or (y=1) then
   begin
   SwapInteger(Acol,Arow);
   exit;
   end;
z:=abs(x-y);
for i:=1 to z do if x<y then Addcol else if x>y then AddRow;
for j:=1 to ARow do for i:=j to Acol do Exchange(i,j,j,i);
for i:=1 to z do if x<y then DeleteRow(RowCount) else if x>y then DeleteCol(ColCount);
end;

procedure TMatrix.LowTriangulation;
var k,j,i,x1:integer;
    x:tcomplex;
begin
if  acol<arow then
  begin
  errorlog:=12; //    -    (    )
  exit;
  end;
ReverseCount:=0;  //  
ReverseBit:=False; //      
for k:=1 to Arow-1 do
 begin
   if ComplexEqualZero(Items[k,k]) then //    =0  
    begin
    x1:=k+1;
    while (x1<Arow) and (ComplexEqualZero(Items[k,x1])) do inc(x1);
    if (x1=Arow) and (ComplexEqualZero(Items[k,x1])) then
      for i:=1 to ARow do exchange(k,i,k+1,i)
    else
    for i:=1 to ACol do exchange(i,k,i,x1);
     inc(ReverseCount);
     ReverseBit:=Not ReverseBit;
    end;
  for j:=k+1 to Arow do
   begin
   x:=DivComplex(NegativeCOmplex(Items[k,j]),Items[k,k]);
   for i:=k to Acol do Items[i,j]:=addcomplex(mulcomplex(Items[i,k],x),Items[i,j]);
   end;
 end;
end;

procedure TMatrix.HighTriangulation; //    
var k,j,i:integer;
    x:tcomplex;
begin
{
if  acol<>arow then
  begin
  errorlog:=11; //  
  exit;
  end;
}
for k:=Arow downto 2 do
 begin
  for j:=k-1 downto 1 do
   begin
   x:=DivComplex(NegativeCOmplex(Items[k,j]),Items[k,k]);
    for i:=Acol downto 1 do
     begin
     Items[i,j]:=addcomplex(mulcomplex(Items[i,k],x),Items[i,j]);
     end;
   end;
 end;
end;

procedure TMatrix.InvertMatrix;
var k,i,j:integer;
    s,r:tcomplex;
//    useprogress:boolean;
//    counter:integer;
begin
if  acol<>arow then
  begin
  raise EMatherror.create('Matrix Not Square.');
  exit;
  end;

//if ( Assigned(fInvertProgress)) then UseProgress:=true else UseProgress:=false;
//counter:=2*Arow+(Acol div 2);

for k:=1 to Arow do //   
 begin
 addcol;
 items[k+arow,k]:=makecomplex(1,0);
end;

for k:=1 to Arow do //  
 begin
  s:=items[k,k];
  j:=k;
  for i:=k+1 to Arow do
   begin
   r:=items[k,i];
   if abs(r.r)>abs(s.r) then
    begin
    s:=r;
    j:=i;
    end;
   end;
{  if useprogress then
     begin
     finvertprogress(self,counter,k+arow);
     Application.ProcessMessages;
     end;}
  if ComplexEqualZero(s) then break;
  if j<>k then for i:=k to Acol do exchange(i,k,i,j);
  for j:=k+1 to ACol do items[j,k]:=divcomplex(items[j,k],s);
  for i:=k+1 to Arow do
    for j:=k+1 to acol do items[j,i]:=subcomplex(items[j,i],mulcomplex(items[j,k],items[k,i]));
end;
if not ComplexEqualZero(s) then
 for j:=Arow+1 to Acol do
  for i:=Arow-1 downto 1 do
   for k:=i+1 to arow do items[j,i]:=subcomplex(items[j,i],mulcomplex(items[j,k],items[k,i]));

for k:=1 to (Acol div 2) do
    deletecol(1); //  

end;

procedure TMatrix.FillMatrixWithValue(z:tcomplex);
var i,j:integer;
begin
for i:=1 to rowcount do for j:=1 to colcount do Items[j,i]:=z;
end;

procedure TMatrix.FillMatrix(mode:integer);
var i,j,x:integer;
begin
{
0:  
1:  
2:  1        1
3:  Acol*Arow        1
4:  
5:   
}
case mode of
0:for i:=1 to ARow do for j:=1 to ACol do Items[j,i]:=Nullcomplex;
1:for i:=1 to ARow do for j:=1 to ACol do if j<>i then Items[j,i]:=Nullcomplex else Items[j,i]:=MakeComplex(1,0);
2:begin
  x:=0;
  for i:=1 to ARow do for j:=1 to ACol do
     begin
     inc(x);
     Items[j,i]:=makecomplex(x,0);
     end;
  end;
3:begin
  x:=Acol*Arow+1;
  for i:=1 to ARow do for j:=1 to ACol do
     begin
     dec(x);
     Items[j,i]:=makecomplex(x,0);
     end;
  end;
4:for i:=1 to ARow do for j:=1 to ACol do Items[j,i]:=MakeComplex(Random(Acol*Arow),0);
5:for i:=1 to ARow do for j:=1 to ACol do Items[j,i]:=MakeComplex(Random(Acol*Arow),Random(Acol*Arow));
end;
end;

function TMatrix.GetDeterminant:Tcomplex;
var t:Tmatrix;
    i,j:integer;
begin

t:=Tmatrix.create(acol,arow);
for j:=1 to arow do for i:=1 to acol do t[i,j]:=Items[i,j];
t.lowtriangulation;
result:=makecomplex(1,0);
for i:=1 to acol do result:=mulcomplex(result,t[i,i]);
if t.ReverseBit=true then result:=NegativeComplex(result);
t.free;

end;

procedure Tmatrix.RecreateMatrix(x,y:integer);
var s,i:integer;
begin
s:=x*y-acol*arow;
for i:=1 to Abs(s) do if s>0 then addzeroitem else AItems.delete(0);
Aitems.Capacity:=x*y;
acol:=x;
arow:=y;
end;

procedure TMatrix.CopyFromMatrix(z:Tmatrix);
var i,j:integer;
begin
if z.Rowcount*z.Colcount<>Acol*arow then RecreateMatrix(z.Colcount,z.Rowcount)
else
 begin
 acol:=z.colcount;
 arow:=z.rowcount;
 end;
for j:=1 to Arow do for i:=1 to acol do Items[i,j]:=z[i,j];
end;

procedure TMatrix.CopyRealFromMatrix(z:Tmatrix);
var i,j:integer;
begin
if z.Rowcount*z.Colcount<>Acol*arow then RecreateMatrix(z.Colcount,z.Rowcount)
else
 begin
 acol:=z.colcount;
 arow:=z.rowcount;
 end;
for j:=1 to Arow do for i:=1 to acol do Items[i,j]:=Makecomplex(z[i,j].r,Items[i,j].i);
end;

procedure TMatrix.CopyImagineFromMatrix(z:Tmatrix);
var i,j:integer;
begin
if z.Rowcount*z.Colcount<>Acol*arow then RecreateMatrix(z.Colcount,z.Rowcount)
else
 begin
 acol:=z.colcount;
 arow:=z.rowcount;
 end;
for j:=1 to Arow do for i:=1 to acol do Items[i,j]:=Makecomplex(Items[i,j].r,z[i,j].i);
end;



procedure TMatrix.FlipVertical;
var j,i:integer;
begin
for j:=1 to (Arow div 2) do  for i:=1 to Acol do exchange(i,j,i,Arow-(j-1));
end;

Procedure TMatrix.FlipHorizontal;
var j,i:integer;
begin
for j:=1 to Arow do for i:=1 to (Acol div 2) do exchange(i,j,Acol-(i-1),j);
end;

function TMatrix.GetCheckPrecision:Boolean;
begin
result:=FCheckPrecision;
end;

procedure TMatrix.SetCheckPrecision(value:Boolean);
begin
FCheckPrecision:=Value;
end;

procedure TMatrix.AddMatrix(m:Tmatrix);
var i,j:integer;
begin
if (m.ColCount=Acol) or (m.rowCount=Arow) then
 begin
 for j:=1 to Arow do for i:=1 to acol do Items[i,j]:=Addcomplex(Items[i,j],m[i,j]);
 end
else
 Raise EMathError.create('Appliyng Matrix Not Same Size');
end;

procedure TMatrix.SubMatrix(m:Tmatrix);
var i,j:integer;
begin
if (m.ColCount=Acol) or (m.rowCount=Arow) then
 begin
 for j:=1 to Arow do for i:=1 to acol do Items[i,j]:=Subcomplex(Items[i,j],m[i,j]);
 end
else
 Raise EMathError.create('Appliyng Matrix Not Same Size');
end;

procedure TMatrix.MulMatrixPerComplex(z:Tcomplex);
var i,j:integer;
begin
for j:=1 to Arow do for i:=1 to acol do Items[i,j]:=Mulcomplex(Items[i,j],z);
end;

procedure TMatrix.DivMatrixPerComplex(z:Tcomplex);
var i,j:integer;
begin
if not ComplexEqualZero(z) then
 begin
 for j:=1 to Arow do for i:=1 to acol do Items[i,j]:=Divcomplex(Items[i,j],z);
 end
else
  raise EDivByZero.Create('Evaluting Can''t Complete Because Deviding By Zero');
end;

function TMatrix.GetFootStep:TComplex; //   
var i:integer;
begin
result:=nullcomplex;
if acol<>arow then Raise EMathError.create('Matrix Not Square.')
else
 for i:=1 to acol do result:=AddComplex(result,Items[i,i]);
end;

Procedure TMatrix.FillMatrixFromArray(const A:array of Tcomplex);
var i,j,x:integer;
begin
if high(a)<>(acol*arow-1) then Raise ERangeError.Create('Comes Short elements in array.')
else
begin
  x:=0;
  for i:=1 to ARow do
  for j:=1 to ACol do
     begin
     Items[j,i]:=a[x];
     inc(x);
     end;
end;
end;

Procedure TMatrix.FillMatrixFromArrayE(const A:array of double);
var i,j,x:integer;
begin
if high(a)<>(acol*arow-1) then Raise ERangeError.Create('Comes Short elements in array.')
else
begin
  x:=0;
  for i:=1 to ARow do
  for j:=1 to ACol do
     begin
     Items[j,i]:=MakeComplex(a[x],0);
     inc(x);
     end;
end;
end;

Procedure TMatrix.FillGridWithMatrix(Grid:TStringGrid;x,y:integer);
var i,j:Integer;
begin
  Grid.ColCount:=ACol+x;
  Grid.RowCount:=ARow+y;
  for j:=1 to RowCount do
   for i:=1 to ColCount do
    Grid.Cells[x+i-1,y+j-1]:=COmplexToStr(Items[i,j]);
end;

Function TMatrix.MaxModuleNonDiagonalItem(var x,y:integer):TComplex; //        
var Temp1:double;
    i,j:integer;
begin
Temp1:=ModuleCOmplex(Items[1,1]);
for j:=1 to RowCount do
 for i:=1 to Colcount do
  if i<>j then if ModuleCOmplex(Items[i,j])>Temp1 then
    begin
    x:=i;
    y:=j;
    Result:=Items[i,j];
    end;
end;

Procedure TMatrix.RotateMatrix(Angle:double;mx,my:integer);
Var c,s:Tcomplex;
    i:integer;
begin
c:=MakeComplex(cos(Angle),0);
s:=MakeCOmplex(Sin(Angle),0);
for i:=1 to RowCount do
  begin
  Items[mx,i]:=SubComplex( MulComplex(Items[mx,i],c) , MulCOmplex(Items[my,i],s) );
  Items[my,i]:=AddCOmplex( MulComplex(Items[mx,i],s) , MulComplex(Items[my,i],c) );
  end;
for i:=1 to ColCount do
  begin
  Items[i,mx]:=SubComplex( MulComplex(Items[i,mx],c) , MulComplex(Items[i,my],s) );
  Items[i,my]:=AddCOmplex( MulComplex(Items[i,mx],s) , MulComplex(Items[i,my],c) );
  end;
end;

procedure TMatrix.InvertSign;
var i,j:integer;
begin
for i:=1 to RowCount do
 for j:=1 to ColCount do
   Items[j,i]:=NegativeComplex(Items[j,i]);
end;

procedure TMatrix.TakeMatrixPart(x,y,x1,y1:integer;var A:TMatrix);
var i,j:integer;
begin
if (x>Colcount) or (x1>Colcount) or (y>Rowcount) or (y1>Rowcount) then exit;

if A=nil then A:=TMatrix.Create(x1-x+1,y1-y+1)
 else if (A.ColCOunt<>(x1-x+1)) or (A.RowCOunt<>(y1-y+1)) then A.RecreateMatrix(x1-x+1,y1-y+1);

for i:=y to y1 do
 for j:=x to x1 do
  A[j-x+1,i-y+1]:=Items[j,i];
end;

procedure TMatrix.PlaceMatrixPart(A:TMatrix;x,y:integer);
var i,j,x1,y1:integer;
begin
x1:=x+A.ColCount;
y1:=y+A.RowCount;

if x1>ColCount then x1:=ColCount;
if y1>ColCount then y1:=RowCount;

for i:=y to y1 do
 for j:=x to x1 do
  Items[j,i]:=A[j-x+1,i-y+1];

end;

function TMatrix.Norma:Double;
var i,j:integer;
begin
result:=0;
for i:=1 to RowCount do
 for j:=1 to ColCount do
  result:=result + ModuleComplex(Items[j,i]);

Result:=SQRT(result);
end;

procedure TMatrix.ExchangeCols(x1,x2:integer);
var i:integer;
begin
for i:=1 to rowcount do Exchange(x1,i,x2,i);
end;

procedure TMatrix.ExchangeRows(y1,y2:integer);
var i:integer;
begin
for i:=1 to colcount do Exchange(i,y1,i,y2);
end;


procedure SolveMatrixL(L,B:Tmatrix;var Z:Tmatrix); //      
var i,j:integer;
begin
z.FillMAtrix(0);
Z[1,1]:=DivComplex(B[1,1],L[1,1]);
for i:=2 to L.RowCount do
begin
 for j:=1 to i-1 do Z[1,i]:=SubComplex(Z[1,i], DivComplex(MulComplex(Z[1,j],L[j,i]),L[i,i]) );
   Z[1,i]:=AddComplex(DivComplex(B[1,i],L[i,i]),Z[1,i]);
end;
end;

procedure SolveMatrixW(W,B:Tmatrix;var Z:Tmatrix); //      
var i,j:integer;
begin
z.FillMAtrix(0);
Z[1,Z.Rowcount]:=DivComplex(B[1,Z.Rowcount],W[Z.Rowcount,Z.Rowcount]);
for i:=Z.Rowcount-1 downto 1 do
begin
 for j:=Z.Rowcount downto i+1 do Z[1,i]:=SubComplex(Z[1,i], DivComplex(MulComplex(Z[1,j],W[j,i]),W[i,i]) );
 Z[1,i]:=AddComplex(DivComplex(B[1,i],W[i,i]),Z[1,i]);
end;
end;


procedure GaussInvert(A:TMatrix;var B:TMatrix);
var i,j:integer;
    l,w,c,z1,z2:TMatrix;

    procedure FillVectorWIthOne(var s:TMatrix;i:integer);
    begin
    s.fillMatrix(0);
    s[1,i]:=MakeComplex(1,0);
    end;
begin
if not EqualSizeMatrix(A,B) then B.RecreateMatrix(A.colcount,A.rowcount);
l:=Tmatrix.Create(A.colcount,A.colcount);
w:=Tmatrix.Create(A.colcount,A.colcount);

c:=Tmatrix.Create(1,A.colcount);
z1:=Tmatrix.Create(1,A.colcount);
z2:=Tmatrix.Create(1,A.colcount);

DeterminateLW(A,l,w);

for i:=1 to A.colcount do
 begin
 FillVectorWithOne(c,i);
 SolveMatrixL(l,c,z1);
 solveMatrixW(w,z1,z2);
 b.PlaceMatrixPart(z2,i,1);
 end;

l.free;
w.free;
c.free;
z1.free;
z2.free;
end;

procedure TMatrix.SaveToStream(AStream:TStream);
var Hdr:TFileHeader;
    i,j:integer;
    z:TComplex;
begin
 with hdr do
  begin
  lbl:=LabelCode;
  Header:=HeaderCode;
  Version:=VersionCode;
  FCol:=ColCount;
  FRow:=RowCount;
  end;
 AStream.Write(hdr,SizeOf(hdr));

 for j:=1 to rowcount do
  for i:=1 to colcount do
    begin
    z:=Items[i,j];
    AStream.Write(z,SizeOf(z));
    end;
end;

procedure TMatrix.SaveToFile(Name:string);
var tmp:TFileStream;
begin
  tmp:=TFileStream.Create(name,fmCreate);
  try
    SaveToStream(tmp);
  finally
    tmp.Free;
  end;
end;

procedure TMAtrix.LoadFromStream(AStream:TStream);
var hdr:TFileHeader;
    i,j:integer;
    Z:Tcomplex;
begin
  AStream.Read(hdr,SizeOf(hdr));

  if hdr.lbl=LabelCode then
   begin
    if hdr.Header=HeaderCode then
     begin
      if hdr.Version=VersionCode then
       begin

       RecreateMatrix(hdr.Fcol,hdr.frow);
       FillMatrix(0);

       for j:=1 to hdr.Frow do
        for i:=1 to hdr.FCol do
         begin
         Astream.read(z,sizeof(z));
         Items[i,j]:=z;
         end;

       end
      else raise Exception.Create('Version of file not supported');
     end
    else raise Exception.Create('Wrong header of matrix');
   end
  else raise Exception.Create('Wrong signature of matrix');

end;

procedure TMatrix.LoadFromFile(Name:string);
var tmp:TFileStream;
begin
  tmp:=TFileStream.Create(name,fmOpenRead);
  try
    LoadFromStream(tmp);
  finally
    tmp.Free;
  end;
end;


end.

